@@ -29,6 +29,7 @@
 enum {
 	ARG_0,
 	ARG_SCALE_FACTOR,
+	ARG_AUTO_RESIZE,
 	ARG_VISIBLE_CURSOR,
 	ARG_LOGO_FOCUSED,
 	ARG_EVENT_CATCHER,
@@ -38,6 +39,30 @@ enum {
 	ARG_LOGO,
 };
 
+struct _GstVideoWidgetPrivate {
+
+	GdkWindow *event_window;
+	GdkWindow *video_window;
+
+	GdkPixbuf *logo_pixbuf;
+	
+	gulong xembed_xid;
+	
+	guint source_width;
+	guint source_height;
+	
+	guint width_mini;
+	guint height_mini;
+	
+	gboolean auto_resize;
+	gboolean cursor_visible;
+	gboolean event_catcher;
+	gboolean logo_focused;
+	
+	gboolean scale_override;
+	gfloat scale_factor;
+};
+
 static GtkWidgetClass *parent_class = NULL;
 
 /* ============================================================= */
@@ -67,7 +92,7 @@ gst_video_widget_update_cursor (GstVideoWidget *vw)
 	if (widget->window == NULL)
 		return;
 	
-	if (vw->cursor_visible)
+	if (vw->priv->cursor_visible)
 		gdk_window_set_cursor (widget->window, NULL);
 	else {
 	
@@ -106,20 +131,21 @@ gst_video_widget_reorder_windows (GstVideoWidget *vw)
 	g_return_if_fail(vw != NULL);
 	g_return_if_fail(GST_IS_VIDEO_WIDGET(vw));
 	
-	if (vw->event_catcher) {
-		if (GDK_IS_WINDOW(vw->event_window))
-			gdk_window_raise(vw->event_window);
+	if (vw->priv->event_catcher) {
+		if (GDK_IS_WINDOW(vw->priv->event_window))
+			gdk_window_raise(vw->priv->event_window);
 	}
 	else {
-		if (GDK_IS_WINDOW(vw->video_window))
-			gdk_window_raise(vw->video_window);
+		if (GDK_IS_WINDOW(vw->priv->video_window))
+			gdk_window_raise(vw->priv->video_window);
 	}
 	
-	if ( (vw->logo_focused) && (GDK_IS_WINDOW(vw->video_window)) ) {
-		gdk_window_hide(vw->video_window);
+	if ( (vw->priv->logo_focused) && (GDK_IS_WINDOW(vw->priv->video_window)) ) {
+		gdk_window_hide(vw->priv->video_window);
 	}
-	else if ( (!vw->logo_focused) && (GDK_IS_WINDOW(vw->video_window)) ) {
-		gdk_window_show(vw->video_window);
+	else if (	(!vw->priv->logo_focused) &&
+				(GDK_IS_WINDOW(vw->priv->video_window)) ) {
+		gdk_window_show(vw->priv->video_window);
 	}
 	else {
 		gtk_widget_queue_draw (GTK_WIDGET(vw));
@@ -177,13 +203,13 @@ gst_video_widget_realize (GtkWidget *widget)
 	
 	attributes_mask = GDK_WA_X | GDK_WA_Y;
 
-  	vw->event_window = gdk_window_new (
+  	vw->priv->event_window = gdk_window_new (
 								widget->window,
 								&attributes, attributes_mask);
 								
-	gdk_window_set_user_data (vw->event_window, widget);
+	gdk_window_set_user_data (vw->priv->event_window, widget);
 	
-	gdk_window_show (vw->event_window);
+	gdk_window_show (vw->priv->event_window);
 	
 	widget->style = gtk_style_attach (widget->style, widget->window);
 	
@@ -216,16 +242,16 @@ gst_video_widget_unrealize (GtkWidget *widget)
 
 	/* Destroying event window */
 	
-	if (GDK_IS_WINDOW(vw->event_window)) {
-		gdk_window_set_user_data(vw->event_window, NULL);
-		gdk_window_destroy(vw->event_window);
-		vw->event_window = NULL;
+	if (GDK_IS_WINDOW(vw->priv->event_window)) {
+		gdk_window_set_user_data(vw->priv->event_window, NULL);
+		gdk_window_destroy(vw->priv->event_window);
+		vw->priv->event_window = NULL;
 	}
 	
-	if (GDK_IS_WINDOW(vw->video_window)) {
-		gdk_window_set_user_data(vw->video_window, NULL);
-		gdk_window_destroy(vw->video_window);
-		vw->video_window = NULL;
+	if (GDK_IS_WINDOW(vw->priv->video_window)) {
+		gdk_window_set_user_data(vw->priv->video_window, NULL);
+		gdk_window_destroy(vw->priv->video_window);
+		vw->priv->video_window = NULL;
 	}
 	
 	/* Chaining up */
@@ -244,12 +270,14 @@ gst_video_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 	g_return_val_if_fail(widget != NULL, FALSE);
 	g_return_val_if_fail(GST_IS_VIDEO_WIDGET(widget), FALSE);
 	g_return_val_if_fail(event != NULL, FALSE);
-	
+
 	vw = GST_VIDEO_WIDGET (widget);
+	
+	/* g_message ("expose %d,%d",event->area.width, event->area.height);*/
 		
 	if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget)) {
 		
-		if ( (vw->logo_focused) &&  (vw->logo_pixbuf) ) {
+		if ( (vw->priv->logo_focused) &&  (vw->priv->logo_pixbuf) ) {
 			GdkPixbuf *frame;
 			guchar *pixels;
 			int rowstride;
@@ -260,8 +288,8 @@ gst_video_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 							FALSE, 8, widget->allocation.width,
 							widget->allocation.height);
 			
-			width = gdk_pixbuf_get_width (vw->logo_pixbuf);
-			height = gdk_pixbuf_get_height (vw->logo_pixbuf);
+			width = gdk_pixbuf_get_width (vw->priv->logo_pixbuf);
+			height = gdk_pixbuf_get_height (vw->priv->logo_pixbuf);
 			alloc_width = widget->allocation.width;
 			alloc_height = widget->allocation.height;
 			
@@ -282,7 +310,7 @@ gst_video_widget_expose(GtkWidget *widget, GdkEventExpose *event)
 			/* Scaling to available space */
 			
 			gdk_pixbuf_composite (
-					vw->logo_pixbuf,
+					vw->priv->logo_pixbuf,
 					frame,
 					0, 0, 
 					alloc_width, alloc_height,
@@ -334,10 +362,18 @@ gst_video_widget_size_request (GtkWidget *widget, GtkRequisition *requisition)
 	
 	vw = GST_VIDEO_WIDGET (widget);
 	
-	if ( (vw->source_width) && (vw->source_height) && (vw->scale_factor) ) {
-		temp = (vw->scale_factor * vw->source_width + 0.5);
+	if ( (!vw->priv->auto_resize) && (!vw->priv->scale_override) ){
+		requisition->width = vw->priv->width_mini;
+		requisition->height = vw->priv->height_mini;
+		return;
+	}
+	
+	if (	(vw->priv->source_width) &&
+			(vw->priv->source_height) &&
+			(vw->priv->scale_factor) ) {
+		temp = (vw->priv->scale_factor * vw->priv->source_width + 0.5);
 		width = (gint) temp > G_MAXINT ? G_MAXINT : (gint) temp;
-		temp = (vw->scale_factor * vw->source_height + 0.5);
+		temp = (vw->priv->scale_factor * vw->priv->source_height + 0.5);
 		height = (gint) temp > G_MAXINT ? G_MAXINT : (gint) temp;
 	
 		/* don't make us want to be bigger than the screen */
@@ -352,9 +388,11 @@ gst_video_widget_size_request (GtkWidget *widget, GtkRequisition *requisition)
 		}
 	}
 	else {
-		if (vw->logo_pixbuf) {
-			width = gdk_pixbuf_get_width(vw->logo_pixbuf);
-			height = gdk_pixbuf_get_height(vw->logo_pixbuf);
+		if (vw->priv->logo_pixbuf) {
+			width = gdk_pixbuf_get_width(vw->priv->logo_pixbuf);
+			height = gdk_pixbuf_get_height(vw->priv->logo_pixbuf);
+			vw->priv->width_mini = width;
+			vw->priv->height_mini = height;
 		}
 		else {
 			width = 100;
@@ -362,6 +400,11 @@ gst_video_widget_size_request (GtkWidget *widget, GtkRequisition *requisition)
 		}
 	}
 	
+	if (width < vw->priv->width_mini)
+		width = vw->priv->width_mini;
+	if (height < vw->priv->height_mini)
+		height = vw->priv->height_mini;
+		
 	requisition->width = width;
 	requisition->height = height;
 }
@@ -372,45 +415,70 @@ static void
 gst_video_widget_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
 	GstVideoWidget *vw;
-	gfloat width_ratio, height_ratio, temp, scale_factor;
+	gfloat width_ratio, height_ratio, temp, scale_factor = 1.0;
 	guint width, height;
 
 	g_return_if_fail(widget != NULL);
 	g_return_if_fail(GST_IS_VIDEO_WIDGET(widget));
 	
 	vw = GST_VIDEO_WIDGET (widget);
-
-	widget->allocation = *allocation;
 	
 	/* Choosing best ratio */
 	
-	if (vw->scale_override) {
-		scale_factor = vw->scale_factor;
-		vw->scale_override = FALSE;
+	if (vw->priv->scale_override) {
+		scale_factor = vw->priv->scale_factor;
 	}
-	else {
+	else if (!vw->priv->auto_resize) {
 		
 		/* Ratio get impacted only if video window loaded */
-		if (	(vw->source_width) &&
-				(vw->source_height) &&
-				(GDK_IS_WINDOW(vw->video_window)) )
+		if (	(vw->priv->source_width) &&
+				(vw->priv->source_height) &&
+				(GDK_IS_WINDOW(vw->priv->video_window)) )
 		{
 			width_ratio = (gfloat) allocation->width /
-							(gfloat) vw->source_width;
+							(gfloat) vw->priv->source_width;
 			height_ratio = (gfloat) allocation->height /
-							(gfloat) vw->source_height;
+							(gfloat) vw->priv->source_height;
 	
 			scale_factor = MIN (width_ratio, height_ratio);
 		}
 	}
+	else {
+		scale_factor = 1.0;
+	}
 	
 	/* Calculating width & height with optimal ratio */
 	
-	temp = (scale_factor * vw->source_width + 0.5);
+	temp = (scale_factor * vw->priv->source_width + 0.5);
 	width = (gint) temp > G_MAXINT ? G_MAXINT : (gint) temp;
-	temp = (scale_factor * vw->source_height + 0.5);
+	temp = (scale_factor * vw->priv->source_height + 0.5);
 	height = (gint) temp > G_MAXINT ? G_MAXINT : (gint) temp;
 
+	/* If auto resize is set we check that video size is
+	bigger than minimum size */
+	if (vw->priv->auto_resize) {
+		if (width < vw->priv->width_mini) {
+			allocation->width = vw->priv->width_mini;
+		}
+		else {
+			allocation->width = width;
+		}
+		if (height < vw->priv->height_mini) {
+			allocation->height = vw->priv->height_mini;
+		}
+		else {
+			allocation->height = height;
+		}
+	} /* If not auto resizing we check that allocation is bigger */
+	else { /* than minimum size */
+		if (allocation->width < vw->priv->width_mini)
+			allocation->width = vw->priv->width_mini;
+		if (allocation->height < vw->priv->height_mini)
+			allocation->height = vw->priv->height_mini;
+	}
+	
+	widget->allocation = *allocation;
+	
 	if (GTK_WIDGET_REALIZED (widget)) {
 		gdk_window_move_resize (	widget->window,
 									allocation->x, 
@@ -418,21 +486,21 @@ gst_video_widget_allocate (GtkWidget *widget, GtkAllocation *allocation)
 									allocation->width, 
 									allocation->height);
 		
-		if (GDK_IS_WINDOW(vw->event_window))
-			gdk_window_move_resize (	vw->event_window,
+		if (GDK_IS_WINDOW(vw->priv->event_window))
+			gdk_window_move_resize (	vw->priv->event_window,
 										0, 
 										0,
 										allocation->width, 
 										allocation->height);
 		
-		if (GDK_IS_WINDOW(vw->video_window)) {
+		if (GDK_IS_WINDOW(vw->priv->video_window)) {
 			gint video_x, video_y;
 						
 			video_x = (allocation->width / 2) - (width / 2);
 			video_y = (allocation->height / 2) - (height / 2);
 					
 			gdk_window_move_resize (
-							vw->video_window,
+							vw->priv->video_window,
 							video_x, 
 							video_y,
 							width, 
@@ -454,30 +522,34 @@ gst_video_widget_set_property (GObject *object, guint prop_id, const GValue *val
 
 	switch (prop_id) {
 		case ARG_SCALE_FACTOR:
-			vw->scale_factor = g_value_get_float (value);
-			vw->scale_override = TRUE;
+			vw->priv->scale_factor = g_value_get_float (value);
+			vw->priv->scale_override = TRUE;
+			gtk_widget_queue_resize (GTK_WIDGET(vw));
+		break;
+		case ARG_AUTO_RESIZE:
+			vw->priv->auto_resize = g_value_get_boolean (value);
 			gtk_widget_queue_resize (GTK_WIDGET(vw));
 		break;
 		case ARG_VISIBLE_CURSOR:
-			vw->cursor_visible = g_value_get_boolean (value);
+			vw->priv->cursor_visible = g_value_get_boolean (value);
 			gst_video_widget_update_cursor (vw);
 		break;
 		case ARG_LOGO_FOCUSED:
-			vw->logo_focused = g_value_get_boolean (value);
+			vw->priv->logo_focused = g_value_get_boolean (value);
 			gst_video_widget_reorder_windows (vw);
 		break;
 		case ARG_EVENT_CATCHER:
-			vw->event_catcher = g_value_get_boolean (value);
+			vw->priv->event_catcher = g_value_get_boolean (value);
 			gst_video_widget_reorder_windows (vw);
 		break;
 		case ARG_XID:
 			gst_video_widget_set_xembed_xid (vw, g_value_get_ulong(value));
 		break;
 		case ARG_SOURCE_WIDTH:
-			vw->source_width = g_value_get_int (value);
+			vw->priv->source_width = g_value_get_int (value);
 		break;
 		case ARG_SOURCE_HEIGHT:
-			vw->source_height = g_value_get_int (value);
+			vw->priv->source_height = g_value_get_int (value);
 		break;
 		case ARG_LOGO:
 		{
@@ -507,29 +579,32 @@ gst_video_widget_get_property (GObject *object, guint prop_id, GValue *value, GP
 
 	switch (prop_id) {
 		case ARG_SCALE_FACTOR:
-			g_value_set_float (value, vw->scale_factor);
+			g_value_set_float (value, vw->priv->scale_factor);
+		break;
+		case ARG_AUTO_RESIZE:
+			g_value_set_boolean (value, vw->priv->auto_resize);
 		break;
 		case ARG_VISIBLE_CURSOR:
-			g_value_set_boolean (value, vw->cursor_visible);
+			g_value_set_boolean (value, vw->priv->cursor_visible);
 		break;
 		case ARG_LOGO_FOCUSED:
-			g_value_set_boolean (value, vw->logo_focused);
+			g_value_set_boolean (value, vw->priv->logo_focused);
 		break;
 		case ARG_EVENT_CATCHER:
-			g_value_set_boolean (value, vw->event_catcher);
+			g_value_set_boolean (value, vw->priv->event_catcher);
 		break;
 		case ARG_XID:
-			g_value_set_ulong (value, vw->xembed_xid);
+			g_value_set_ulong (value, vw->priv->xembed_xid);
 		break;
 		case ARG_SOURCE_WIDTH:
-			g_value_set_int (value, vw->source_width);
+			g_value_set_int (value, vw->priv->source_width);
 		break;
 		case ARG_SOURCE_HEIGHT:
-			g_value_set_int (value, vw->source_height);
+			g_value_set_int (value, vw->priv->source_height);
 		break;
 		case ARG_LOGO:
 			g_value_set_object (value,
-                          (GObject*) vw->logo_pixbuf);
+                          (GObject*) vw->priv->logo_pixbuf);
 		break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -558,7 +633,7 @@ gst_video_widget_class_init (GstVideoWidgetClass *klass)
 	
 	gobject_class->set_property = gst_video_widget_set_property;
 	gobject_class->get_property = gst_video_widget_get_property;
-
+	
 	g_object_class_install_property (
 				gobject_class,
 				ARG_SCALE_FACTOR,
@@ -568,6 +643,15 @@ gst_video_widget_class_init (GstVideoWidgetClass *klass)
 						"size the video should be scaled to",
 						0, G_MAXFLOAT / G_MAXINT, 1, G_PARAM_READWRITE));
 
+	g_object_class_install_property (
+				gobject_class,
+				ARG_AUTO_RESIZE,
+				g_param_spec_boolean (
+						"auto_resize",
+						"auto resize",
+						"Is the video widget resizing automatically",
+						FALSE, G_PARAM_READWRITE));
+	
 	g_object_class_install_property (
 				gobject_class,
 				ARG_VISIBLE_CURSOR,
@@ -645,20 +729,23 @@ static void
 gst_video_widget_init (GstVideoWidget *vw)
 {
 	
+	vw->priv = g_new0 (GstVideoWidgetPrivate, 1);
+	
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET(vw), GTK_CAN_FOCUS);
 	
-	vw->source_width	=	0;
-	vw->source_height	=	0;
-/*	vw->x_align			=	0.5;
-	vw->y_align			=	0.5; */
-	vw->scale_factor	=	1.0;
-	vw->scale_override	=	FALSE;
-	vw->cursor_visible	=	TRUE;
-	vw->event_catcher	=	TRUE;
-	vw->logo_focused	=	TRUE;
-	vw->event_window	=	NULL;
-	vw->video_window	=	NULL;
-	vw->logo_pixbuf		=	NULL;
+	vw->priv->source_width	=	0;
+	vw->priv->source_height	=	0;
+	vw->priv->width_mini	=	0;
+	vw->priv->height_mini	=	0;
+	vw->priv->scale_factor	=	1.0;
+	vw->priv->auto_resize	=	FALSE;
+	vw->priv->scale_override=	FALSE;
+	vw->priv->cursor_visible=	TRUE;
+	vw->priv->event_catcher	=	TRUE;
+	vw->priv->logo_focused	=	TRUE;
+	vw->priv->event_window	=	NULL;
+	vw->priv->video_window	=	NULL;
+	vw->priv->logo_pixbuf	=	NULL;
 }
 
 /* ============================================================= */
@@ -686,25 +773,33 @@ gst_video_widget_set_xembed_xid (GstVideoWidget *vw, gulong xid)
 	
 	gdk_threads_enter ();
 		
-	vw->logo_focused = FALSE;
+	vw->priv->logo_focused = FALSE;
 	
-	if (GDK_IS_WINDOW(vw->video_window)) {
-		gdk_window_destroy (vw->video_window);
-		vw->video_window = NULL;
+	if (GDK_IS_WINDOW(vw->priv->video_window)) {
+		gdk_window_destroy (vw->priv->video_window);
+		vw->priv->video_window = NULL;
 	}
 	
-	vw->video_window = gdk_window_foreign_new (xid);
+	vw->priv->video_window = gdk_window_foreign_new (xid);
 	
-	if (vw->video_window) {
+	if (vw->priv->video_window) {
+		gint video_x, video_y, video_width, video_height, video_depth;
 		
-		gdk_window_reparent (vw->video_window, widget->window, 0, 0);
+		gdk_window_reparent (vw->priv->video_window, widget->window, 0, 0);
 		
-		gdk_window_show (vw->video_window);
+		gdk_window_show (vw->priv->video_window);
 		
-		if (vw->event_catcher)
-			gdk_window_raise (vw->event_window);
+		gdk_window_get_geometry (	vw->priv->video_window,
+									&video_x, &video_y,
+									&video_width, &video_height, &video_depth);
+		
+		vw->priv->source_width = video_width;
+		vw->priv->source_height = video_height;
+		
+		if (vw->priv->event_catcher)
+			gdk_window_raise (vw->priv->event_window);
 	
-		vw->xembed_xid = xid;
+		vw->priv->xembed_xid = xid;
 		
 		gtk_widget_queue_resize (GTK_WIDGET(vw));
 
@@ -730,7 +825,7 @@ gst_video_widget_get_xembed_xid (GstVideoWidget *vw)
 {
 	g_return_val_if_fail(vw != NULL, 0);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), 0);
-	return vw->xembed_xid;
+	return vw->priv->xembed_xid;
 }
 
 /**
@@ -742,7 +837,7 @@ gst_video_widget_get_xembed_xid (GstVideoWidget *vw)
  * Set video source size of a #GstVideoWidget and queue a resize request
  *  for the widget.
  *
- * The #GstVideoWidget must have already be created
+ * The #GstVideoWidget must have already been created
  *  before you can make this call.
  * 
  * Remember you can set these values trough "source_width" and "source_height"
@@ -756,8 +851,8 @@ gst_video_widget_set_source_size (GstVideoWidget *vw, gint width, gint height)
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
 	
-	vw->source_width = width;
-	vw->source_height = height;
+	vw->priv->source_width = width;
+	vw->priv->source_height = height;
 	gtk_widget_queue_resize (GTK_WIDGET(vw));
 	
 	return TRUE;
@@ -782,8 +877,64 @@ gst_video_widget_get_source_size (GstVideoWidget *vw, gint *width, gint *height)
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
 
-	*width = vw->source_width;
-	*height = vw->source_height;
+	*width = vw->priv->source_width;
+	*height = vw->priv->source_height;
+	
+	return TRUE;
+}
+
+/**
+ * gst_video_widget_set_minimum_size:
+ * @vw: a #GstVideoWidget
+ * @width: a #gint indicating minimum width.
+ * @height: a #gint indicating minimum height.
+ * 
+ * Set minimum size of a #GstVideoWidget and queue a resize request
+ *  for the widget. This method is usefull when the #GstVideoWidget is set
+ *  to auto resize, it won't go under this size.
+ *
+ * The #GstVideoWidget must have already been created
+ *  before you can make this call.
+ * 
+ * Remember you can set these values trough "width_mini" and "height_mini"
+ *  properties, but you will have to queue the resize request yourself.
+ * 
+ * Return value: a #gboolean indicating wether the call succeeded or not.
+ **/
+gboolean
+gst_video_widget_set_minimum_size (GstVideoWidget *vw, gint width, gint height)
+{
+	g_return_val_if_fail(vw != NULL, FALSE);
+	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
+	
+	vw->priv->width_mini = width;
+	vw->priv->height_mini = height;
+	gtk_widget_queue_resize (GTK_WIDGET(vw));
+	
+	return TRUE;
+}
+
+/**
+ * gst_video_widget_get_minimum_size:
+ * @socket_: a #GstVideoWidget
+ * @width: a pointer to a #gint where minimum width will be put.
+ * @height: a pointer to a #gint where minimum height will be put.
+ * 
+ * Fills @width and @height with minimum dimensions.
+ *
+ * Remember you can get these values trough "width_mini" and "height_mini"
+ *  properties.
+ *
+ * Return value: a #gboolean indicating wether the call succeeded or not.
+ **/
+gboolean
+gst_video_widget_get_minimum_size (GstVideoWidget *vw, gint *width, gint *height)
+{
+	g_return_val_if_fail(vw != NULL, FALSE);
+	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
+
+	*width = vw->priv->width_mini;
+	*height = vw->priv->height_mini;
 	
 	return TRUE;
 }
@@ -805,7 +956,7 @@ gst_video_widget_set_cursor_visible (GstVideoWidget *vw, gboolean visible)
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
 	
-	vw->cursor_visible = visible;
+	vw->priv->cursor_visible = visible;
 	
 	gst_video_widget_update_cursor (vw);
 	
@@ -827,7 +978,7 @@ gst_video_widget_get_cursor_visible (GstVideoWidget *vw)
 {
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-	return vw->cursor_visible;
+	return vw->priv->cursor_visible;
 }
 
 /**
@@ -847,7 +998,7 @@ gst_video_widget_set_logo_focus (GstVideoWidget *vw, gboolean focused)
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
 	
-	vw->logo_focused = focused;
+	vw->priv->logo_focused = focused;
 	
 	gst_video_widget_reorder_windows (vw);
 	
@@ -869,7 +1020,7 @@ gst_video_widget_get_logo_focus (GstVideoWidget *vw)
 {
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-	return vw->logo_focused;
+	return vw->priv->logo_focused;
 }
 
 /**
@@ -890,7 +1041,7 @@ gst_video_widget_set_event_catcher (GstVideoWidget *vw, gboolean event_catcher)
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
 	
-	vw->event_catcher = event_catcher;
+	vw->priv->event_catcher = event_catcher;
 	
 	gst_video_widget_reorder_windows (vw);
 	
@@ -913,7 +1064,87 @@ gst_video_widget_get_event_catcher (GstVideoWidget *vw)
 {
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-	return vw->event_catcher;
+	return vw->priv->event_catcher;
+}
+
+/**
+ * gst_video_widget_set_auto_resize:
+ * @vw: a #GstVideoWidget
+ * @resize: a #gboolean indicating auto resize mode that will be used by @vw.
+ * 
+ * Set if a #GstVideoWidget will auto resize or not.
+ *
+ * Remember you can set this flag trough the "auto_resize" property.
+ *
+ * Return value: a #gboolean indicating wether the call succeeded or not.
+ **/
+gboolean
+gst_video_widget_set_auto_resize (GstVideoWidget *vw, gboolean resize)
+{
+	g_return_val_if_fail(vw != NULL, FALSE);
+	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
+	vw->priv->auto_resize = resize;
+	gtk_widget_queue_resize (GTK_WIDGET(vw));
+	return TRUE;
+}
+
+/**
+ * gst_video_widget_get_auto_resize:
+ * @vw: a #GstVideoWidget
+ * 
+ * Get used auto resize mode for a #GstVideoWidget.
+ *
+ * Remember you can get this value trough "auto_resize" property.
+ *
+ * Return value: a #gboolean indicating wether the video widget
+ * is in auto_resize mode or not.
+ **/
+gboolean
+gst_video_widget_get_auto_resize (GstVideoWidget *vw)
+{
+	g_return_val_if_fail(vw != NULL, FALSE);
+	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
+	return vw->priv->auto_resize;
+}
+
+/**
+ * gst_video_widget_get_scale_override:
+ * @vw: a #GstVideoWidget
+ * 
+ * Get scale override mode for a #GstVideoWidget.
+ *
+ * Remember you can get this value trough "scale_override" property.
+ *
+ * Return value: a #gboolean indicating if scale ratio is enforced or not.
+ **/
+gboolean
+gst_video_widget_get_scale_override (GstVideoWidget *vw)
+{
+	g_return_val_if_fail(vw != NULL, FALSE);
+	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
+	return vw->priv->scale_override;
+}
+
+/**
+ * gst_video_widget_set_scale_override:
+ * @vw: a #GstVideoWidget
+ * @override: a #gboolean indicating scale override mode that will
+ * be used by @vw.
+ * 
+ * Set scale override mode for a #GstVideoWidget.
+ *
+ * Remember you can set this flag trough the "scale_override" property.
+ *
+ * Return value: a #gboolean indicating wether the call succeeded or not.
+ **/
+gboolean
+gst_video_widget_set_scale_override (GstVideoWidget *vw, gboolean override)
+{
+	g_return_val_if_fail(vw != NULL, FALSE);
+	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
+	vw->priv->scale_override = override;
+	gtk_widget_queue_resize (GTK_WIDGET(vw));
+	return TRUE;
 }
 
 /**
@@ -932,9 +1163,9 @@ gst_video_widget_set_scale (GstVideoWidget *vw, gfloat scale)
 {
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-	vw->scale_factor = scale;
-	vw->scale_override = TRUE;
-	gtk_widget_queue_resize (GTK_WIDGET(vw));
+	vw->priv->scale_factor = scale;
+	if (vw->priv->scale_override)
+		gtk_widget_queue_resize (GTK_WIDGET(vw));
 	return TRUE;
 }
 
@@ -953,7 +1184,7 @@ gst_video_widget_get_scale (GstVideoWidget *vw)
 {
 	g_return_val_if_fail(vw != NULL, FALSE);
 	g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-	return vw->scale_factor;
+	return vw->priv->scale_factor;
 }
 
 /** 
@@ -972,13 +1203,13 @@ gst_video_widget_set_logo (GstVideoWidget *vw, GdkPixbuf *logo_pixbuf)
 	g_return_if_fail (vw != NULL);
 	g_return_if_fail (GST_IS_VIDEO_WIDGET(vw));
 
-	if (logo_pixbuf == vw->logo_pixbuf)
+	if (logo_pixbuf == vw->priv->logo_pixbuf)
 		return;
 
-	if (vw->logo_pixbuf)
-		g_object_unref (vw->logo_pixbuf);
+	if (vw->priv->logo_pixbuf)
+		g_object_unref (vw->priv->logo_pixbuf);
 
-	vw->logo_pixbuf = logo_pixbuf;
+	vw->priv->logo_pixbuf = logo_pixbuf;
 }
 
 /** 
@@ -993,7 +1224,7 @@ gst_video_widget_get_logo (GstVideoWidget *vw)
 {
 	g_return_if_fail (vw != NULL);
 	g_return_if_fail (GST_IS_VIDEO_WIDGET(vw));
-	return vw->logo_pixbuf;
+	return vw->priv->logo_pixbuf;
 }
 
 /* =========================================== */
