@@ -181,6 +181,7 @@ gtk_playlist_mrl_to_title (const gchar *mrl)
 
 	filename = g_path_get_basename (mrl);
 	unescaped = gnome_vfs_unescape_string_for_display (filename);
+
 	g_free (filename);
 	filename_for_display = g_filename_to_utf8 (unescaped,
 			-1,             /* length */
@@ -188,6 +189,14 @@ gtk_playlist_mrl_to_title (const gchar *mrl)
 			NULL,           /* bytes_written */
 			NULL);          /* error */
 
+	if (filename_for_display == NULL)
+	{
+		filename_for_display = g_locale_to_utf8 (unescaped,
+				-1, NULL, NULL, NULL);
+		if (filename_for_display == NULL)
+			return unescaped;
+	}
+
 	g_free (unescaped);
 
 	return filename_for_display;
@@ -431,6 +440,23 @@ my_eel_read_entire_file (const char *uri,
 	return GNOME_VFS_OK;
 }
 
+static char*
+gtk_playlist_base_url (const char *mrl)
+{
+	/* Yay, let's reconstruct the base by hand */
+	GnomeVFSURI *uri, *parent;
+	char *base;
+
+	uri = gnome_vfs_uri_new (mrl);
+	parent = gnome_vfs_uri_get_parent (uri);
+	base = gnome_vfs_uri_to_string (parent, 0);
+
+	gnome_vfs_uri_unref (uri);
+	gnome_vfs_uri_unref (parent);
+
+	return base;
+}
+
 static int
 read_ini_line_int (char **lines, const char *key)
 {
@@ -1318,6 +1344,7 @@ gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl, gpointer data)
 	gboolean retval = FALSE;
 	char *contents, **lines;
 	int size, i;
+	char *split_char, *base;
 
 	if (my_eel_read_entire_file (mrl, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
@@ -1325,12 +1352,18 @@ gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl, gpointer data)
 	contents = g_realloc (contents, size + 1);
 	contents[size] = '\0';
 
-	lines = g_strsplit (contents, "\n", 0);
+	/* figure out whether we're a unix m3u or dos m3u */
+	if (strstr(contents,"\x0d") == NULL)
+		split_char = "\n";
+	else
+		split_char = "\x0d\n";
+
+	lines = g_strsplit (contents, split_char, 0);
 	g_free (contents);
 
 	for (i = 0; lines[i] != NULL; i++)
 	{
-		/* Either it's a URI, or it has a proper path */
+		/* Either it's a URI, or it has a proper path ... */
 		if (strstr(lines[i], "://") != NULL
 				|| lines[i][0] == G_DIR_SEPARATOR)
 		{
@@ -1339,6 +1372,35 @@ gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl, gpointer data)
 			if (gtk_playlist_add_mrl (playlist,
 						lines[i], NULL) == TRUE)
 				retval = TRUE;
+		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
+			/* ... Or it's in the windows smb form
+			 * (\\machine\share\filename), Note drive names
+			 * (C:\ D:\ etc) are unhandled (unknown base for
+			 * drive letters) */
+		        char *tmpmrl;
+
+			lines[i] = g_strdelimit (lines[i], "\\", '/');
+			tmpmrl = g_strjoin (NULL, "smb:", lines[i], NULL);
+
+			if (gtk_playlist_add_one_mrl
+					(playlist, tmpmrl, NULL) == TRUE)
+				retval = TRUE;
+
+			g_free (tmpmrl);
+		} else {
+			/* Try with a base */
+			char *fullpath, *base, sep;
+
+			base = gtk_playlist_base_url (mrl);
+			sep = (split_char[0] == '\n' ? '/' : '\\');
+			if (sep == '\\')
+				lines[i] = g_strdelimit (lines[i], "\\", '/');
+			fullpath = g_strdup_printf ("%s/%s", base, lines[i]);
+			if (gtk_playlist_add_mrl (playlist,
+						fullpath, NULL) == TRUE)
+				retval = TRUE;
+			g_free (fullpath);
+			g_free (base);
 		}
 	}
 
@@ -1542,16 +1604,7 @@ gtk_playlist_add_asx (GtkPlaylist *playlist, const char *mrl, gpointer data)
 		return FALSE;
 	}
 
-	/* Yay, let's reconstruct the base by hand */
-	{
-		GnomeVFSURI *uri, *parent;
-		uri = gnome_vfs_uri_new (mrl);
-		parent = gnome_vfs_uri_get_parent (uri);
-		base = gnome_vfs_uri_to_string (parent, 0);
-
-		gnome_vfs_uri_unref (uri);
-		gnome_vfs_uri_unref (parent);
-	}
+	base = gtk_playlist_base_url (mrl);
 
 	for (node = doc->children; node != NULL; node = node->next)
 	{
@@ -1700,16 +1753,7 @@ gtk_playlist_add_smil (GtkPlaylist *playlist, const char *mrl, gpointer data)
 		return FALSE;
 	}
 
-	/* Yay, let's reconstruct the base by hand */
-	{
-		GnomeVFSURI *uri, *parent;
-		uri = gnome_vfs_uri_new (mrl);
-		parent = gnome_vfs_uri_get_parent (uri);
-		base = gnome_vfs_uri_to_string (parent, 0);
-
-		gnome_vfs_uri_unref (uri);
-		gnome_vfs_uri_unref (parent);
-	}
+	base = gtk_playlist_base_url (mrl);
 
 	for (node = doc->children; node != NULL; node = node->next)
 	{
@@ -1788,6 +1832,7 @@ static PlaylistTypes dual_types[] = {
 	{ "audio/x-pn-realaudio", gtk_playlist_add_ra },
 	{ "application/vnd.rn-realmedia", gtk_playlist_add_ra },
 	{ "audio/x-pn-realaudio-plugin", gtk_playlist_add_ra },
+	{ "text/plain", gtk_playlist_add_ra },
 	{ "video/x-ms-asf", gtk_playlist_add_asf },
 	{ "video/x-ms-wmv", gtk_playlist_add_asf },
 };
